@@ -4,25 +4,19 @@ use pyo3::prelude::*;
 
 macro_rules! test_module {
     ($py:ident, $code:literal) => {
-        PyModule::from_code($py, indoc::indoc!($code), file!(), "test_module")
-            .expect("module creation failed")
+        PyModule::from_code($py, $code, file!(), "test_module").expect("module creation failed")
     };
 }
 
 fn bench_call_0(b: &mut Bencher) {
     Python::with_gil(|py| {
-        let module = test_module!(
-            py,
-            r#"
-            def foo(): pass
-        "#
-        );
+        let module = test_module!(py, "def foo(): pass");
 
-        let foo = module.getattr("foo").unwrap();
+        let foo_module = module.getattr("foo").unwrap();
 
         b.iter(|| {
             for _ in 0..1000 {
-                foo.call0().unwrap();
+                foo_module.call0().unwrap();
             }
         });
     })
@@ -32,17 +26,18 @@ fn bench_call_method_0(b: &mut Bencher) {
     Python::with_gil(|py| {
         let module = test_module!(
             py,
-            r#"
-            class Foo:
-                def foo(self): pass
-        "#
+            "
+class Foo:
+    def foo(self):
+        pass
+"
         );
 
-        let foo = module.getattr("Foo").unwrap().call0().unwrap();
+        let foo_module = module.getattr("Foo").unwrap().call0().unwrap();
 
         b.iter(|| {
             for _ in 0..1000 {
-                foo.call_method0("foo").unwrap();
+                foo_module.call_method0("foo").unwrap();
             }
         });
     })
