@@ -1,6 +1,5 @@
-use criterion::{criterion_group, criterion_main, Bencher, Criterion};
-
-use pyo3::{class::PyObjectProtocol, prelude::*, type_object::LazyStaticType};
+use criterion::{criterion_group, criterion_main, Criterion};
+use pyo3::{prelude::*, type_object::LazyStaticType};
 
 /// This is a feature-rich class instance used to benchmark various parts of the pyclass lifecycle.
 #[pyclass]
@@ -16,22 +15,18 @@ impl MyClass {
         Self { elements }
     }
 
-    #[call]
-    fn call(&mut self, new_element: i32) -> usize {
+    fn __call__(&mut self, new_element: i32) -> usize {
         self.elements.push(new_element);
         self.elements.len()
     }
-}
 
-#[pyproto]
-impl PyObjectProtocol for MyClass {
     /// A basic __str__ implementation.
     fn __str__(&self) -> &'static str {
         "MyClass"
     }
 }
 
-fn first_time_init(b: &mut Bencher) {
+pub fn first_time_init(b: &mut criterion::Bencher) {
     let gil = Python::acquire_gil();
     let py = gil.python();
     b.iter(|| {
@@ -47,4 +42,5 @@ fn criterion_benchmark(c: &mut Criterion) {
 }
 
 criterion_group!(benches, criterion_benchmark);
+
 criterion_main!(benches);
