@@ -2,7 +2,8 @@
 
 //! Interaction with Python's global interpreter lock
 
-use crate::{ffi, internal_tricks::Unsendable, Python};
+use crate::impl_::not_send::{NotSend, NOT_SEND};
+use crate::{ffi, Python};
 use parking_lot::{const_mutex, Mutex, Once};
 use std::cell::{Cell, RefCell};
 use std::{
@@ -45,20 +46,12 @@ pub(crate) fn gil_is_acquired() -> bool {
 /// signal handling depends on the notion of a 'main thread', which must be the thread that
 /// initializes the Python interpreter.
 ///
-/// If both the Python interpreter and Python threading are already initialized, this function has
-/// no effect.
+/// If the Python interpreter is already initialized, this function has no effect.
 ///
 /// This function is unavailable under PyPy because PyPy cannot be embedded in Rust (or any other
 /// software). Support for this is tracked on the
 /// [PyPy issue tracker](https://foss.heptapod.net/pypy/pypy/-/issues/3286).
 ///
-/// Python 3.6 only: If the Python interpreter is initialized but Python threading is not, this
-/// function will initialize Python threading.
-///
-/// # Panics
-/// - Python 3.6 only: If this function needs to initialize Python threading but the calling thread
-///   is not the thread which initialized Python, this function will panic.
-///
 /// # Examples
 /// ```rust
 /// use pyo3::prelude::*;
@@ -69,33 +62,13 @@ pub(crate) fn gil_is_acquired() -> bool {
 /// # }
 /// ```
 #[cfg(not(PyPy))]
-#[allow(clippy::collapsible_if)] // for if cfg!
 pub fn prepare_freethreaded_python() {
     // Protect against race conditions when Python is not yet initialized and multiple threads
     // concurrently call 'prepare_freethreaded_python()'. Note that we do not protect against
     // concurrent initialization of the Python runtime by other users of the Python C API.
     START.call_once_force(|_| unsafe {
-        if cfg!(not(Py_3_7)) {
-            // Use call_once_force because if initialization panics, it's okay to try again.
-            if ffi::Py_IsInitialized() != 0 {
-                if ffi::PyEval_ThreadsInitialized() == 0 {
-                    // We can only safely initialize threads if this thread holds the GIL.
-                    assert!(
-                        !ffi::PyGILState_GetThisThreadState().is_null(),
-                        "Python threading is not initialized and cannot be initialized by this \
-                         thread, because it is not the thread which initialized Python."
-                    );
-                    ffi::PyEval_InitThreads();
-                }
-            } else {
-                ffi::Py_InitializeEx(0);
-                ffi::PyEval_InitThreads();
-
-                // Release the GIL.
-                ffi::PyEval_SaveThread();
-            }
-        } else if ffi::Py_IsInitialized() == 0 {
-            // In Python 3.7 and up PyEval_InitThreads is irrelevant.
+        // Use call_once_force because if initialization panics, it's okay to try again.
+        if ffi::Py_IsInitialized() == 0 {
             ffi::Py_InitializeEx(0);
 
             // Release the GIL.
@@ -134,7 +107,6 @@ pub fn prepare_freethreaded_python() {
 /// # }
 /// ```
 #[cfg(not(PyPy))]
-#[allow(clippy::collapsible_if)] // for if cfg!
 pub unsafe fn with_embedded_python_interpreter<F, R>(f: F) -> R
 where
     F: for<'p> FnOnce(Python<'p>) -> R,
@@ -147,15 +119,7 @@ where
 
     ffi::Py_InitializeEx(0);
 
-    // Changed in version 3.7: This function is now called by Py_Initialize(), so you don’t have to
-    // call it yourself anymore.
-    if cfg!(not(Py_3_7)) {
-        if ffi::PyEval_ThreadsInitialized() == 0 {
-            ffi::PyEval_InitThreads();
-        }
-    }
-
-    // Safe: the GIL is already held because of the Py_IntializeEx call.
+    // Safety: the GIL is already held because of the Py_IntializeEx call.
     let pool = GILPool::new();
 
     // Import the threading module - this ensures that it will associate this thread as the "main"
@@ -189,11 +153,12 @@ where
 ///     let py = gil_guard.python();
 /// } // GIL is released when gil_guard is dropped
 /// ```
-#[allow(clippy::upper_case_acronyms)]
+
 #[must_use]
 pub struct GILGuard {
     gstate: ffi::PyGILState_STATE,
     pool: ManuallyDrop<Option<GILPool>>,
+    _not_send: NotSend,
 }
 
 impl GILGuard {
@@ -218,6 +183,15 @@ impl GILGuard {
             if #[cfg(all(feature = "auto-initialize", not(PyPy)))] {
                 prepare_freethreaded_python();
             } else {
+                // This is a "hack" to make running `cargo test` for PyO3 convenient (i.e. no need
+                // to specify `--features auto-initialize` manually. Tests within the crate itself
+                // all depend on the auto-initialize feature for conciseness but Cargo does not
+                // provide a mechanism to specify required features for tests.
+                #[cfg(not(PyPy))]
+                if option_env!("CARGO_PRIMARY_PACKAGE").is_some() {
+                    prepare_freethreaded_python();
+                }
+
                 START.call_once_force(|_| unsafe {
                     // Use call_once_force because if there is a panic because the interpreter is
                     // not initialized, it's fine for the user to initialize the interpreter and
@@ -230,14 +204,6 @@ impl GILGuard {
                          Consider calling `pyo3::prepare_freethreaded_python()` before attempting \
                          to use Python APIs."
                     );
-                    assert_ne!(
-                        ffi::PyEval_ThreadsInitialized(),
-                        0,
-                        "Python threading is not initalized and the `auto-initialize` feature is \
-                         not enabled.\n\n\
-                         Consider calling `pyo3::prepare_freethreaded_python()` before attempting \
-                         to use Python APIs."
-                    );
                 });
             }
         }
@@ -266,6 +232,7 @@ impl GILGuard {
         GILGuard {
             gstate,
             pool: ManuallyDrop::new(pool),
+            _not_send: NOT_SEND,
         }
     }
 }
@@ -274,6 +241,7 @@ impl GILGuard {
 impl Drop for GILGuard {
     fn drop(&mut self) {
         // First up, try to detect if the order of destruction is correct.
+        #[allow(clippy::manual_assert)]
         let _ = GIL_COUNT.try_with(|c| {
             if self.gstate == ffi::PyGILState_STATE::PyGILState_UNLOCKED && c.get() != 1 {
                 // XXX: this panic commits to leaking all objects in the pool as well as
@@ -363,12 +331,11 @@ static POOL: ReferencePool = ReferencePool::new();
 
 ///
 /// [Memory Management]: https://pyo3.rs/main/memory.html#gil-bound-memory
-#[allow(clippy::upper_case_acronyms)]
 pub struct GILPool {
     /// Initial length of owned objects and anys.
     /// `Option` is used since TSL can be broken when `new` is called from `atexit`.
     start: Option<usize>,
-    no_send: Unsendable,
+    _not_send: NotSend,
 }
 
 impl GILPool {
@@ -387,11 +354,12 @@ impl GILPool {
         POOL.update_counts(Python::assume_gil_acquired());
         GILPool {
             start: OWNED_OBJECTS.try_with(|o| o.borrow().len()).ok(),
-            no_send: Unsendable::default(),
+            _not_send: NOT_SEND,
         }
     }
 
     /// Gets the Python token associated with this [`GILPool`].
+    #[inline]
     pub fn python(&self) -> Python {
         unsafe { Python::assume_gil_acquired() }
     }
@@ -505,7 +473,6 @@ pub(crate) fn ensure_gil_unchecked() -> EnsureGIL {
 }
 
 /// Struct used internally which avoids acquiring the GIL where it's not necessary.
-#[allow(clippy::upper_case_acronyms)]
 pub(crate) struct EnsureGIL(Option<GILGuard>);
 
 impl EnsureGIL {
@@ -709,10 +676,9 @@ mod tests {
         // Acquiring GIL for the second time should be safe - see #864
         let gil = Python::acquire_gil();
         let py = gil.python();
-        let obj;
 
         let gil2 = Python::acquire_gil();
-        obj = py.eval("object()", None, None).unwrap();
+        let obj = py.eval("object()", None, None).unwrap();
         drop(gil2);
 
         // After gil2 drops, obj should still have a reference count of one
@@ -735,71 +701,119 @@ mod tests {
 
     #[test]
     fn test_clone_without_gil() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let obj = get_object(py);
-        let count = obj.get_refcnt(py);
+        use crate::{Py, PyAny};
+        use std::sync::atomic::{AtomicBool, Ordering};
+        use std::{sync::Arc, thread};
 
-        // Cloning without GIL should not update reference count
-        drop(gil);
-        let c = obj.clone();
-        assert_eq!(
-            count,
-            obj.get_refcnt(unsafe { Python::assume_gil_acquired() })
-        );
+        // Some spinlocks for synchronizing
+        static GIL_ACQUIRED: AtomicBool = AtomicBool::new(false);
+        static OBJECT_CLONED: AtomicBool = AtomicBool::new(false);
+        static REFCNT_CHECKED: AtomicBool = AtomicBool::new(false);
 
-        // Acquring GIL will clear this pending change
-        let gil = Python::acquire_gil();
-        let py = gil.python();
+        Python::with_gil(|py| {
+            let obj: Arc<Py<PyAny>> = Arc::new(get_object(py));
+            let thread_obj = Arc::clone(&obj);
 
-        // Total reference count should be one higher
-        assert_eq!(count + 1, obj.get_refcnt(py));
+            let count = (&*obj).get_refcnt(py);
+            println!(
+                "1: The object has been created and its reference count is {}",
+                count
+            );
 
-        // Clone dropped
-        drop(c);
+            let handle = thread::spawn(move || {
+                Python::with_gil(move |py| {
+                    println!("3. The GIL has been acquired on another thread.");
+                    GIL_ACQUIRED.store(true, Ordering::Release);
 
-        // Overall count is now back to the original, and should be no pending change
-        assert_eq!(count, obj.get_refcnt(py));
+                    // Spin a bit while the main thread registers obj in POOL
+                    while !OBJECT_CLONED.load(Ordering::Acquire) {}
+                    println!("5. Checking refcnt");
+                    assert_eq!(thread_obj.get_refcnt(py), count);
+
+                    REFCNT_CHECKED.store(true, Ordering::Release);
+                })
+            });
+
+            let cloned = py.allow_threads(|| {
+                println!("2. The GIL has been released.");
+
+                // spin until the gil has been acquired on the thread.
+                while !GIL_ACQUIRED.load(Ordering::Acquire) {}
+
+                println!("4. The other thread is now hogging the GIL, we clone without it held");
+                // Cloning without GIL should not update reference count
+                let cloned = Py::clone(&*obj);
+                OBJECT_CLONED.store(true, Ordering::Release);
+                cloned
+            });
+
+            while !REFCNT_CHECKED.load(Ordering::Acquire) {}
+
+            // Returning from allow_threads doesn't clear the pool
+            py.allow_threads(|| {
+                // Acquiring GIL will clear the pending change
+                Python::with_gil(|_| {});
+            });
+
+            println!("6. The main thread has acquired the GIL again and processed the pool.");
+
+            // Total reference count should be one higher
+            assert_eq!(obj.get_refcnt(py), count + 1);
+
+            // Clone dropped
+            drop(cloned);
+            // Ensure refcount of the arc is 1
+            handle.join().unwrap();
+
+            // Overall count is now back to the original, and should be no pending change
+            assert_eq!(Arc::try_unwrap(obj).unwrap().get_refcnt(py), count);
+        });
     }
 
     #[test]
     fn test_clone_in_other_thread() {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let obj = get_object(py);
-        let count = obj.get_refcnt(py);
+        use crate::Py;
+        use std::sync::atomic::{AtomicBool, Ordering};
+        use std::{sync::Arc, thread};
 
-        // Move obj to a thread which does not have the GIL, and clone it
-        let t = std::thread::spawn(move || {
-            // Cloning without GIL should not update reference count
-            #[allow(clippy::redundant_clone)]
-            let _ = obj.clone();
-            assert_eq!(
-                count,
-                obj.get_refcnt(unsafe { Python::assume_gil_acquired() })
-            );
+        // Some spinlocks for synchronizing
+        static OBJECT_CLONED: AtomicBool = AtomicBool::new(false);
 
-            // Return obj so original thread can continue to use
-            obj
+        let (obj, count, ptr) = Python::with_gil(|py| {
+            let obj = Arc::new(get_object(py));
+            let count = obj.get_refcnt(py);
+            let thread_obj = Arc::clone(&obj);
+
+            // Start a thread which does not have the GIL, and clone it
+            let t = thread::spawn(move || {
+                // Cloning without GIL should not update reference count
+                #[allow(clippy::redundant_clone)]
+                let _ = Py::clone(&*thread_obj);
+                OBJECT_CLONED.store(true, Ordering::Release);
+            });
+
+            while !OBJECT_CLONED.load(Ordering::Acquire) {}
+            assert_eq!(count, obj.get_refcnt(py));
+
+            t.join().unwrap();
+            let ptr = NonNull::new(obj.as_ptr()).unwrap();
+
+            // The pointer should appear once in the incref pool, and once in the
+            // decref pool (for the clone being created and also dropped)
+            assert!(POOL.pointer_ops.lock().0.contains(&ptr));
+            assert!(POOL.pointer_ops.lock().1.contains(&ptr));
+
+            (obj, count, ptr)
         });
 
-        let obj = t.join().unwrap();
-        let ptr = NonNull::new(obj.as_ptr()).unwrap();
+        Python::with_gil(|py| {
+            // Acquiring the gil clears the pool
+            assert!(!POOL.pointer_ops.lock().0.contains(&ptr));
+            assert!(!POOL.pointer_ops.lock().1.contains(&ptr));
 
-        // The pointer should appear once in the incref pool, and once in the
-        // decref pool (for the clone being created and also dropped)
-        assert_eq!(&POOL.pointer_ops.lock().0, &vec![ptr]);
-        assert_eq!(&POOL.pointer_ops.lock().1, &vec![ptr]);
-
-        // Re-acquring GIL will clear these pending changes
-        drop(gil);
-        let gil = Python::acquire_gil();
-
-        assert!(POOL.pointer_ops.lock().0.is_empty());
-        assert!(POOL.pointer_ops.lock().1.is_empty());
-
-        // Overall count is still unchanged
-        assert_eq!(count, obj.get_refcnt(gil.python()));
+            // Overall count is still unchanged
+            assert_eq!(count, obj.get_refcnt(py));
+        });
     }
 
     #[test]
