@@ -1,4 +1,4 @@
-#![cfg_attr(feature = "nightly", feature(specialization))]
+#![cfg_attr(feature = "nightly", feature(auto_traits, negative_impls))]
 #![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
 #![cfg_attr(
     docsrs, // rustdoc:: is not supported on msrv
@@ -11,6 +11,7 @@
 // Deny some lints in doctests.
 // Use `#[allow(...)]` locally to override.
 #![doc(test(attr(deny(warnings), allow(unused_variables, unused_assignments))))]
+#![cfg_attr(coverage, feature(no_coverage))] // used in src/test_hygiene.rs
 
 //! Rust bindings to the Python interpreter.
 //!
@@ -61,7 +62,10 @@
 //! ## Default feature flags
 //!
 //! The following features are turned on by default:
-//! - `macros`: Enables various macros, including all the attribute macros.
+//! - `macros`: Enables various macros, including all the attribute macros excluding the deprecated
+//! `#[pyproto]` attribute.
+//! - `pyproto`: Adds the deprecated `#[pyproto]` attribute macro. Likely to become optional and
+//! then removed in the future.
 //!
 //! ## Optional feature flags
 //!
@@ -93,16 +97,14 @@
 //!
 //! ## Unstable features
 //!
-//! - `nightly`: Gates some optimizations that rely on
-//! [`#![feature(specialization)]`](https://github.com/rust-lang/rfcs/blob/master/text/1210-impl-specialization.md),
-//! for which you'd also need nightly Rust. You should not use this feature.
+//! - `nightly`: Uses  `#![feature(auto_traits, negative_impls)]` to define [`Ungil`] as an auto trait.
 //
 //! ## `rustc` environment flags
 //!
 //! PyO3 uses `rustc`'s `--cfg` flags to enable or disable code used for different Python versions.
 //! If you want to do this for your own crate, you can do so with the [`pyo3-build-config`] crate.
 //!
-//! - `Py_3_6`, `Py_3_7`, `Py_3_8`, `Py_3_9`, `Py_3_10`: Marks code that is only enabled when
+//! - `Py_3_7`, `Py_3_8`, `Py_3_9`, `Py_3_10`: Marks code that is only enabled when
 //!  compiling for a given minimum Python version.
 //! - `Py_LIMITED_API`: Marks code enabled when the `abi3` feature flag is enabled.
 //! - `PyPy` - Marks code enabled when compiling for PyPy.
@@ -110,8 +112,8 @@
 //! # Minimum supported Rust and Python versions
 //!
 //! PyO3 supports the following software versions:
-//!   - Python 3.6 and up (CPython and PyPy)
-//!   - Rust 1.41 and up
+//!   - Python 3.7 and up (CPython and PyPy)
+//!   - Rust 1.48 and up
 //!
 //! # Example: Building a native Python module
 //!
@@ -282,6 +284,7 @@
 //! [Python from Rust]: https://github.com/PyO3/pyo3#using-python-from-rust
 //! [Rust from Python]: https://github.com/PyO3/pyo3#using-rust-from-python
 //! [Features chapter of the guide]: https://pyo3.rs/latest/features.html#features-reference "Features Reference - PyO3 user guide"
+//! [`Ungil`]: crate::marker::Ungil
 pub use crate::class::*;
 pub use crate::conversion::{
     AsPyPointer, FromPyObject, FromPyPointer, IntoPy, IntoPyPointer, PyTryFrom, PyTryInto,
@@ -292,18 +295,48 @@ pub use crate::err::{PyDowncastError, PyErr, PyErrArguments, PyResult};
 pub use crate::gil::{prepare_freethreaded_python, with_embedded_python_interpreter};
 pub use crate::gil::{GILGuard, GILPool};
 pub use crate::instance::{Py, PyNativeType, PyObject};
+pub use crate::marker::Python;
 pub use crate::pycell::{PyCell, PyRef, PyRefMut};
 pub use crate::pyclass::PyClass;
 pub use crate::pyclass_init::PyClassInitializer;
-pub use crate::python::{Python, PythonVersionInfo};
 pub use crate::type_object::PyTypeInfo;
 pub use crate::types::PyAny;
+pub use crate::version::PythonVersionInfo;
+
+// Old directory layout, to be rethought?
+#[cfg(not(feature = "pyproto"))]
+pub mod class {
+    #[doc(hidden)]
+    pub use crate::impl_::pymethods as methods;
+
+    pub use self::gc::{PyTraverseError, PyVisit};
+
+    #[doc(hidden)]
+    pub use self::methods::{
+        PyClassAttributeDef, PyGetterDef, PyMethodDef, PyMethodDefType, PyMethodType, PySetterDef,
+    };
+
+    pub mod basic {
+        pub use crate::pyclass::CompareOp;
+    }
+
+    pub mod pyasync {
+        pub use crate::pyclass::{IterANextOutput, PyIterANextOutput};
+    }
+
+    pub mod iter {
+        pub use crate::pyclass::{IterNextOutput, PyIterNextOutput};
+    }
+
+    pub mod gc {
+        pub use crate::impl_::pymethods::{PyTraverseError, PyVisit};
+    }
+}
 
 #[cfg(feature = "macros")]
 #[doc(hidden)]
 pub use {
     indoc,    // Re-exported for py_run
-    paste,    // Re-exported for wrap_function
     unindent, // Re-exported for py_run
 };
 
@@ -317,6 +350,7 @@ mod internal_tricks;
 pub mod buffer;
 #[doc(hidden)]
 pub mod callback;
+#[cfg(feature = "pyproto")]
 pub mod class;
 pub mod conversion;
 mod conversions;
@@ -330,6 +364,7 @@ mod gil;
 #[doc(hidden)]
 pub mod impl_;
 mod instance;
+pub mod marker;
 pub mod marshal;
 pub mod once_cell;
 pub mod panic;
@@ -337,10 +372,10 @@ pub mod prelude;
 pub mod pycell;
 pub mod pyclass;
 pub mod pyclass_init;
-pub mod pyclass_slots;
-mod python;
+
 pub mod type_object;
 pub mod types;
+mod version;
 
 pub use crate::conversions::*;
 
@@ -351,15 +386,27 @@ pub use crate::conversions::*;
 )]
 #[cfg(feature = "macros")]
 pub mod proc_macro {
-    pub use pyo3_macros::{pyclass, pyfunction, pymethods, pymodule, pyproto};
+    #[cfg(feature = "pyproto")]
+    pub use pyo3_macros::pyproto;
+    pub use pyo3_macros::{pyclass, pyfunction, pymethods, pymodule};
 }
 
+#[cfg(all(feature = "macros", feature = "pyproto"))]
+pub use pyo3_macros::pyproto;
 #[cfg(feature = "macros")]
-pub use pyo3_macros::{pyclass, pyfunction, pymethods, pymodule, pyproto, FromPyObject};
+pub use pyo3_macros::{
+    pyclass, pyfunction, pymethods, pymodule, wrap_pyfunction, wrap_pymodule, FromPyObject,
+};
 
+#[cfg(feature = "macros")]
 #[macro_use]
 mod macros;
 
+/// Test macro hygiene - this is in the crate since we won't have
+/// `pyo3` available in the crate root.
+#[cfg(all(test, feature = "macros"))]
+mod test_hygiene;
+
 /// Test readme and user guide
 #[cfg(doctest)]
 pub mod doc_test {
@@ -370,46 +417,42 @@ pub mod doc_test {
         };
     }
 
-    macro_rules! doctest {
-        ($path:expr, $mod:ident) => {
-            doctest_impl!(include_str!(concat!("../", $path)), $mod);
+    macro_rules! doctests {
+        ($($path:expr => $mod:ident),* $(,)?) => {
+            $(doctest_impl!(include_str!(concat!("../", $path)), $mod);)*
         };
     }
 
-    doctest!("README.md", readme_md);
-    doctest!("guide/src/advanced.md", guide_advanced_md);
-    doctest!(
-        "guide/src/building_and_distribution.md",
-        guide_building_and_distribution_md
-    );
-    doctest!("guide/src/class.md", guide_class_md);
-    doctest!("guide/src/class/protocols.md", guide_class_protocols_md);
-    doctest!("guide/src/conversions.md", guide_conversions_md);
-    doctest!(
-        "guide/src/conversions/tables.md",
-        guide_conversions_tables_md
-    );
+    doctests! {
+        "README.md" => readme_md,
+        "guide/src/advanced.md" => guide_advanced_md,
+        "guide/src/building_and_distribution.md" => guide_building_and_distribution_md,
+        "guide/src/building_and_distribution/multiple_python_versions.md" => guide_bnd_multiple_python_versions_md,
+        "guide/src/class.md" => guide_class_md,
+        "guide/src/class/call.md" => guide_class_call,
+        "guide/src/class/object.md" => guide_class_object,
+        "guide/src/class/numeric.md" => guide_class_numeric,
+        "guide/src/class/protocols.md" => guide_class_protocols_md,
+        "guide/src/conversions.md" => guide_conversions_md,
+        "guide/src/conversions/tables.md" => guide_conversions_tables_md,
+        "guide/src/conversions/traits.md" => guide_conversions_traits_md,
+        "guide/src/debugging.md" => guide_debugging_md,
 
-    doctest!(
-        "guide/src/conversions/traits.md",
-        guide_conversions_traits_md
-    );
-    doctest!("guide/src/debugging.md", guide_debugging_md);
-    doctest!("guide/src/exception.md", guide_exception_md);
-    doctest!("guide/src/function.md", guide_function_md);
-    doctest!("guide/src/migration.md", guide_migration_md);
-    doctest!("guide/src/module.md", guide_module_md);
-    doctest!("guide/src/parallelism.md", guide_parallelism_md);
-    doctest!("guide/src/python_from_rust.md", guide_python_from_rust_md);
-    doctest!("guide/src/rust_cpython.md", guide_rust_cpython_md);
-    doctest!("guide/src/trait_bounds.md", guide_trait_bounds_md);
-    doctest!("guide/src/types.md", guide_types_md);
-    doctest!("guide/src/faq.md", faq);
-    doctest!(
-        "guide/src/python_typing_hints.md",
-        guide_python_typing_hints
-    );
+        // deliberate choice not to test guide/ecosystem because those pages depend on external
+        // crates such as pyo3_asyncio.
 
-    // deliberate choice not to test guide/ecosystem because those pages depend on external crates
-    // such as pyo3_asyncio.
+        "guide/src/exception.md" => guide_exception_md,
+        "guide/src/faq.md" => guide_faq_md,
+        "guide/src/features.md" => guide_features_md,
+        "guide/src/function.md" => guide_function_md,
+        "guide/src/memory.md" => guide_memory_md,
+        "guide/src/migration.md" => guide_migration_md,
+        "guide/src/module.md" => guide_module_md,
+        "guide/src/parallelism.md" => guide_parallelism_md,
+        "guide/src/python_from_rust.md" => guide_python_from_rust_md,
+        "guide/src/python_typing_hints.md" => guide_python_typing_hints_md,
+        "guide/src/rust_cpython.md" => guide_rust_cpython_md,
+        "guide/src/trait_bounds.md" => guide_trait_bounds_md,
+        "guide/src/types.md" => guide_types_md,
+    }
 }
