@@ -5,10 +5,9 @@
 
 use crate::err::PyResult;
 use crate::ffi;
-#[cfg(PyPy)]
-use crate::ffi::datetime::{PyDateTime_FromTimestamp, PyDate_FromTimestamp};
-use crate::ffi::PyDateTimeAPI;
-use crate::ffi::{PyDateTime_Check, PyDate_Check, PyDelta_Check, PyTZInfo_Check, PyTime_Check};
+use crate::ffi::{
+    PyDateTime_CAPI, PyDateTime_FromTimestamp, PyDateTime_IMPORT, PyDate_FromTimestamp,
+};
 #[cfg(not(PyPy))]
 use crate::ffi::{PyDateTime_DATE_GET_FOLD, PyDateTime_TIME_GET_FOLD};
 use crate::ffi::{
@@ -26,15 +25,83 @@ use crate::ffi::{
 use crate::types::PyTuple;
 use crate::{AsPyPointer, PyAny, PyObject, Python, ToPyObject};
 use std::os::raw::c_int;
-#[cfg(not(PyPy))]
-use std::ptr;
 
-/// Access traits
+fn ensure_datetime_api(_py: Python) -> &'static PyDateTime_CAPI {
+    unsafe {
+        if pyo3_ffi::PyDateTimeAPI().is_null() {
+            PyDateTime_IMPORT()
+        }
+
+        &*pyo3_ffi::PyDateTimeAPI()
+    }
+}
+
+// Type Check macros
+//
+// These are bindings around the C API typecheck macros, all of them return
+// `1` if True and `0` if False. In all type check macros, the argument (`op`)
+// must not be `NULL`. The implementations here all call ensure_datetime_api
+// to ensure that the PyDateTimeAPI is initalized before use
+//
+//
+// # Safety
+//
+// These functions must only be called when the GIL is held!
+
+macro_rules! ffi_fun_with_autoinit {
+    ($(#[$outer:meta] unsafe fn $name: ident($arg: ident: *mut PyObject) -> $ret: ty;)*) => {
+        $(
+            #[$outer]
+            #[allow(non_snake_case)]
+            /// # Safety
+            ///
+            /// Must only be called while the GIL is held
+            unsafe fn $name($arg: *mut crate::ffi::PyObject) -> $ret {
+
+                let _ = ensure_datetime_api(Python::assume_gil_acquired());
+                crate::ffi::$name($arg)
+            }
+        )*
+
+
+    };
+}
+
+ffi_fun_with_autoinit! {
+    /// Check if `op` is a `PyDateTimeAPI.DateType` or subtype.
+    unsafe fn PyDate_Check(op: *mut PyObject) -> c_int;
+
+    /// Check if `op` is a `PyDateTimeAPI.DateTimeType` or subtype.
+    unsafe fn PyDateTime_Check(op: *mut PyObject) -> c_int;
+
+    /// Check if `op` is a `PyDateTimeAPI.TimeType` or subtype.
+    unsafe fn PyTime_Check(op: *mut PyObject) -> c_int;
+
+    /// Check if `op` is a `PyDateTimeAPI.DetaType` or subtype.
+    unsafe fn PyDelta_Check(op: *mut PyObject) -> c_int;
+
+    /// Check if `op` is a `PyDateTimeAPI.TZInfoType` or subtype.
+    unsafe fn PyTZInfo_Check(op: *mut PyObject) -> c_int;
+}
+
+// Access traits
 
 /// Trait for accessing the date components of a struct containing a date.
 pub trait PyDateAccess {
+    /// Returns the year, as a positive int.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_GET_YEAR>
     fn get_year(&self) -> i32;
+    /// Returns the month, as an int from 1 through 12.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_GET_MONTH>
     fn get_month(&self) -> u8;
+    /// Returns the day, as an int from 1 through 31.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_GET_DAY>
     fn get_day(&self) -> u8;
 }
 
@@ -44,17 +111,51 @@ pub trait PyDateAccess {
 /// microsecond) representation of the delta, they are *not* intended as
 /// aliases for calculating the total duration in each of these units.
 pub trait PyDeltaAccess {
+    /// Returns the number of days, as an int from -999999999 to 999999999.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DELTA_GET_DAYS>
     fn get_days(&self) -> i32;
+    /// Returns the number of seconds, as an int from 0 through 86399.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DELTA_GET_DAYS>
     fn get_seconds(&self) -> i32;
+    /// Returns the number of microseconds, as an int from 0 through 999999.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DELTA_GET_DAYS>
     fn get_microseconds(&self) -> i32;
 }
 
 /// Trait for accessing the time components of a struct containing a time.
 pub trait PyTimeAccess {
+    /// Returns the hour, as an int from 0 through 23.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DATE_GET_HOUR>
     fn get_hour(&self) -> u8;
+    /// Returns the minute, as an int from 0 through 59.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DATE_GET_MINUTE>
     fn get_minute(&self) -> u8;
+    /// Returns the second, as an int from 0 through 59.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DATE_GET_SECOND>
     fn get_second(&self) -> u8;
+    /// Returns the microsecond, as an int from 0 through 999999.
+    ///
+    /// Implementations should conform to the upstream documentation:
+    /// <https://docs.python.org/3/c-api/datetime.html#c.PyDateTime_DATE_GET_MICROSECOND>
     fn get_microsecond(&self) -> u32;
+    /// Returns whether this date is the later of two moments with the
+    /// same representation, during a repeated interval.
+    ///
+    /// This typically occurs at the end of daylight savings time, or during
+    /// leap seconds. Only valid if the represented time is ambiguous. See
+    /// [PEP 495](https://www.python.org/dev/peps/pep-0495/) for more detail.
     #[cfg(not(PyPy))]
     fn get_fold(&self) -> bool;
 }
@@ -65,19 +166,20 @@ pub struct PyDate(PyAny);
 pyobject_native_type!(
     PyDate,
     crate::ffi::PyDateTime_Date,
-    *PyDateTimeAPI.DateType,
+    *ensure_datetime_api(Python::assume_gil_acquired()).DateType,
     #module=Some("datetime"),
     #checkfunction=PyDate_Check
 );
 
 impl PyDate {
+    /// Creates a new `datetime.date`.
     pub fn new(py: Python, year: i32, month: u8, day: u8) -> PyResult<&PyDate> {
         unsafe {
-            let ptr = (PyDateTimeAPI.Date_FromDate)(
+            let ptr = (ensure_datetime_api(py).Date_FromDate)(
                 year,
                 c_int::from(month),
                 c_int::from(day),
-                PyDateTimeAPI.DateType,
+                ensure_datetime_api(py).DateType,
             );
             py.from_owned_ptr_or_err(ptr)
         }
@@ -89,14 +191,11 @@ impl PyDate {
     pub fn from_timestamp(py: Python, timestamp: i64) -> PyResult<&PyDate> {
         let time_tuple = PyTuple::new(py, &[timestamp]);
 
+        // safety ensure that the API is loaded
+        let _api = ensure_datetime_api(py);
+
         unsafe {
-            #[cfg(PyPy)]
             let ptr = PyDate_FromTimestamp(time_tuple.as_ptr());
-
-            #[cfg(not(PyPy))]
-            let ptr =
-                (PyDateTimeAPI.Date_FromTimestamp)(PyDateTimeAPI.DateType, time_tuple.as_ptr());
-
             py.from_owned_ptr_or_err(ptr)
         }
     }
@@ -122,7 +221,7 @@ pub struct PyDateTime(PyAny);
 pyobject_native_type!(
     PyDateTime,
     crate::ffi::PyDateTime_DateTime,
-    *PyDateTimeAPI.DateTimeType,
+    *ensure_datetime_api(Python::assume_gil_acquired()).DateType,
     #module=Some("datetime"),
     #checkfunction=PyDateTime_Check
 );
@@ -140,8 +239,9 @@ impl PyDateTime {
         microsecond: u32,
         tzinfo: Option<&PyObject>,
     ) -> PyResult<&'p PyDateTime> {
+        let api = ensure_datetime_api(py);
         unsafe {
-            let ptr = (PyDateTimeAPI.DateTime_FromDateAndTime)(
+            let ptr = (api.DateTime_FromDateAndTime)(
                 year,
                 c_int::from(month),
                 c_int::from(day),
@@ -150,7 +250,7 @@ impl PyDateTime {
                 c_int::from(second),
                 microsecond as c_int,
                 opt_to_pyobj(py, tzinfo),
-                PyDateTimeAPI.DateTimeType,
+                api.DateTimeType,
             );
             py.from_owned_ptr_or_err(ptr)
         }
@@ -172,8 +272,9 @@ impl PyDateTime {
         tzinfo: Option<&PyObject>,
         fold: bool,
     ) -> PyResult<&'p PyDateTime> {
+        let api = ensure_datetime_api(py);
         unsafe {
-            let ptr = (PyDateTimeAPI.DateTime_FromDateAndTimeAndFold)(
+            let ptr = (api.DateTime_FromDateAndTimeAndFold)(
                 year,
                 c_int::from(month),
                 c_int::from(day),
@@ -183,7 +284,7 @@ impl PyDateTime {
                 microsecond as c_int,
                 opt_to_pyobj(py, tzinfo),
                 c_int::from(fold),
-                PyDateTimeAPI.DateTimeType,
+                api.DateTimeType,
             );
             py.from_owned_ptr_or_err(ptr)
         }
@@ -206,19 +307,11 @@ impl PyDateTime {
 
         let args = PyTuple::new(py, &[timestamp, time_zone_info]);
 
+        // safety ensure API is loaded
+        let _api = ensure_datetime_api(py);
+
         unsafe {
-            #[cfg(PyPy)]
             let ptr = PyDateTime_FromTimestamp(args.as_ptr());
-
-            #[cfg(not(PyPy))]
-            let ptr = {
-                (PyDateTimeAPI.DateTime_FromTimestamp)(
-                    PyDateTimeAPI.DateTimeType,
-                    args.as_ptr(),
-                    ptr::null_mut(),
-                )
-            };
-
             py.from_owned_ptr_or_err(ptr)
         }
     }
@@ -267,12 +360,13 @@ pub struct PyTime(PyAny);
 pyobject_native_type!(
     PyTime,
     crate::ffi::PyDateTime_Time,
-    *PyDateTimeAPI.TimeType,
+    *ensure_datetime_api(Python::assume_gil_acquired()).TimeType,
     #module=Some("datetime"),
     #checkfunction=PyTime_Check
 );
 
 impl PyTime {
+    /// Creates a new `datetime.time` object.
     pub fn new<'p>(
         py: Python<'p>,
         hour: u8,
@@ -281,14 +375,15 @@ impl PyTime {
         microsecond: u32,
         tzinfo: Option<&PyObject>,
     ) -> PyResult<&'p PyTime> {
+        let api = ensure_datetime_api(py);
         unsafe {
-            let ptr = (PyDateTimeAPI.Time_FromTime)(
+            let ptr = (api.Time_FromTime)(
                 c_int::from(hour),
                 c_int::from(minute),
                 c_int::from(second),
                 microsecond as c_int,
                 opt_to_pyobj(py, tzinfo),
-                PyDateTimeAPI.TimeType,
+                api.TimeType,
             );
             py.from_owned_ptr_or_err(ptr)
         }
@@ -305,15 +400,16 @@ impl PyTime {
         tzinfo: Option<&PyObject>,
         fold: bool,
     ) -> PyResult<&'p PyTime> {
+        let api = ensure_datetime_api(py);
         unsafe {
-            let ptr = (PyDateTimeAPI.Time_FromTimeAndFold)(
+            let ptr = (api.Time_FromTimeAndFold)(
                 c_int::from(hour),
                 c_int::from(minute),
                 c_int::from(second),
                 microsecond as c_int,
                 opt_to_pyobj(py, tzinfo),
                 fold as c_int,
-                PyDateTimeAPI.TimeType,
+                api.TimeType,
             );
             py.from_owned_ptr_or_err(ptr)
         }
@@ -351,7 +447,7 @@ pub struct PyTzInfo(PyAny);
 pyobject_native_type!(
     PyTzInfo,
     crate::ffi::PyObject,
-    *PyDateTimeAPI.TZInfoType,
+    *ensure_datetime_api(Python::assume_gil_acquired()).TZInfoType,
     #module=Some("datetime"),
     #checkfunction=PyTZInfo_Check
 );
@@ -362,12 +458,13 @@ pub struct PyDelta(PyAny);
 pyobject_native_type!(
     PyDelta,
     crate::ffi::PyDateTime_Delta,
-    *PyDateTimeAPI.DeltaType,
+    *ensure_datetime_api(Python::assume_gil_acquired()).DeltaType,
     #module=Some("datetime"),
     #checkfunction=PyDelta_Check
 );
 
 impl PyDelta {
+    /// Creates a new `timedelta`.
     pub fn new(
         py: Python,
         days: i32,
@@ -375,13 +472,14 @@ impl PyDelta {
         microseconds: i32,
         normalize: bool,
     ) -> PyResult<&PyDelta> {
+        let api = ensure_datetime_api(py);
         unsafe {
-            let ptr = (PyDateTimeAPI.Delta_FromDelta)(
+            let ptr = (api.Delta_FromDelta)(
                 days as c_int,
                 seconds as c_int,
                 microseconds as c_int,
                 normalize as c_int,
-                PyDateTimeAPI.DeltaType,
+                api.DeltaType,
             );
             py.from_owned_ptr_or_err(ptr)
         }
@@ -416,8 +514,8 @@ mod tests {
     #[cfg(not(PyPy))]
     #[test]
     fn test_new_with_fold() {
-        pyo3::Python::with_gil(|py| {
-            use pyo3::types::{PyDateTime, PyTimeAccess};
+        crate::Python::with_gil(|py| {
+            use crate::types::{PyDateTime, PyTimeAccess};
 
             let a = PyDateTime::new_with_fold(py, 2021, 1, 23, 20, 32, 40, 341516, None, false);
             let b = PyDateTime::new_with_fold(py, 2021, 1, 23, 20, 32, 40, 341516, None, true);
