@@ -1,10 +1,10 @@
 // Copyright (c) 2017-present PyO3 Project and Contributors
 
 use crate::err::{self, PyDowncastError, PyErr, PyResult};
-use crate::ffi;
 use crate::internal_tricks::get_ssize_index;
 use crate::types::{PyAny, PyList, PyTuple};
-use crate::AsPyPointer;
+use crate::{ffi, PyNativeType};
+use crate::{AsPyPointer, IntoPyPointer, Py, Python};
 use crate::{FromPyObject, PyTryFrom, ToBorrowedObject};
 
 /// Represents a reference to a Python object supporting the sequence protocol.
@@ -269,35 +269,9 @@ impl<'a, T> FromPyObject<'a> for Vec<T>
 where
     T: FromPyObject<'a>,
 {
-    #[cfg(not(feature = "nightly"))]
     fn extract(obj: &'a PyAny) -> PyResult<Self> {
         extract_sequence(obj)
     }
-    #[cfg(feature = "nightly")]
-    default fn extract(obj: &'a PyAny) -> PyResult<Self> {
-        extract_sequence(obj)
-    }
-}
-
-#[cfg(all(feature = "nightly", not(Py_LIMITED_API)))]
-impl<'source, T> FromPyObject<'source> for Vec<T>
-where
-    for<'a> T: FromPyObject<'a> + crate::buffer::Element,
-{
-    fn extract(obj: &'source PyAny) -> PyResult<Self> {
-        // first try buffer protocol
-        if let Ok(buf) = crate::buffer::PyBuffer::get(obj) {
-            if buf.dimensions() == 1 {
-                if let Ok(v) = buf.to_vec(obj.py()) {
-                    buf.release(obj.py());
-                    return Ok(v);
-                }
-            }
-            buf.release(obj.py());
-        }
-        // fall back to sequence protocol
-        extract_sequence(obj)
-    }
 }
 
 fn extract_sequence<'s, T>(obj: &'s PyAny) -> PyResult<Vec<T>>
@@ -335,12 +309,36 @@ impl<'v> PyTryFrom<'v> for PySequence {
     }
 }
 
+impl Py<PySequence> {
+    /// Borrows a GIL-bound reference to the PySequence. By binding to the GIL lifetime, this
+    /// allows the GIL-bound reference to not require `Python` for any of its methods.
+    ///
+    /// ```
+    /// # use pyo3::prelude::*;
+    /// # use pyo3::types::{PyList, PySequence};
+    /// # Python::with_gil(|py| {
+    /// let seq: Py<PySequence> = PyList::empty(py).as_sequence().into();
+    /// let seq: &PySequence = seq.as_ref(py);
+    /// assert_eq!(seq.len().unwrap(), 0);
+    /// # });
+    /// ```
+    pub fn as_ref<'py>(&'py self, _py: Python<'py>) -> &'py PySequence {
+        let any = self.as_ptr() as *const PyAny;
+        unsafe { PyNativeType::unchecked_downcast(&*any) }
+    }
+
+    /// Similar to [`as_ref`](#method.as_ref), and also consumes this `Py` and registers the
+    /// Python object reference in PyO3's object storage. The reference count for the Python
+    /// object will not be decreased until the GIL lifetime ends.
+    pub fn into_ref(self, py: Python) -> &PySequence {
+        unsafe { py.from_owned_ptr(self.into_ptr()) }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::types::{PyList, PySequence};
-    use crate::AsPyPointer;
-    use crate::Python;
-    use crate::{PyObject, PyTryFrom, ToPyObject};
+    use crate::{AsPyPointer, Py, PyObject, PyTryFrom, Python, ToPyObject};
 
     fn get_object() -> PyObject {
         // Convenience function for getting a single unique object
@@ -727,11 +725,11 @@ mod tests {
             let seq = ob.cast_as::<PySequence>(py).unwrap();
             let rep_seq = seq.in_place_repeat(3).unwrap();
             assert_eq!(6, seq.len().unwrap());
-            assert_eq!(seq, rep_seq);
+            assert!(seq.is(rep_seq));
 
             let conc_seq = seq.in_place_concat(seq).unwrap();
             assert_eq!(12, seq.len().unwrap());
-            assert_eq!(seq, conc_seq);
+            assert!(seq.is(conc_seq));
         });
     }
 
@@ -818,4 +816,26 @@ mod tests {
             assert!(seq_from.list().is_ok());
         });
     }
+
+    #[test]
+    fn test_as_ref() {
+        Python::with_gil(|py| {
+            let seq: Py<PySequence> = PyList::empty(py).as_sequence().into();
+            let seq_ref: &PySequence = seq.as_ref(py);
+            assert_eq!(seq_ref.len().unwrap(), 0);
+        })
+    }
+
+    #[test]
+    fn test_into_ref() {
+        Python::with_gil(|py| {
+            let bare_seq = PyList::empty(py).as_sequence();
+            assert_eq!(bare_seq.get_refcnt(), 1);
+            let seq: Py<PySequence> = bare_seq.into();
+            assert_eq!(bare_seq.get_refcnt(), 2);
+            let seq_ref = seq.into_ref(py);
+            assert_eq!(seq_ref.len().unwrap(), 0);
+            assert_eq!(seq_ref.get_refcnt(), 2);
+        })
+    }
 }
