@@ -1,7 +1,8 @@
+#![cfg(feature = "macros")]
+
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyString, PyTuple};
-use pyo3::PyMappingProtocol;
 
 #[macro_use]
 mod common;
@@ -37,8 +38,8 @@ pub struct PyA {
     foo: Option<String>,
 }
 
-#[pyproto]
-impl PyMappingProtocol for PyA {
+#[pymethods]
+impl PyA {
     fn __getitem__(&self, key: String) -> pyo3::PyResult<String> {
         if key == "t" {
             Ok("bar".into())
@@ -194,6 +195,7 @@ struct PyBaz {
 }
 
 #[derive(Debug, FromPyObject)]
+#[allow(dead_code)]
 struct Baz<U, T> {
     e: E<U, T>,
     tup: Tuple,
@@ -309,8 +311,6 @@ pub enum Foo<'a> {
         #[pyo3(item("foo"))]
         a: String,
     },
-    #[pyo3(transparent)]
-    CatchAll(&'a PyAny),
 }
 
 #[pyclass]
@@ -378,15 +378,52 @@ fn test_enum() {
             Foo::StructWithGetItemArg { a } => assert_eq!(a, "test"),
             _ => panic!("Expected extracting Foo::StructWithGetItemArg, got {:?}", f),
         }
+    });
+}
 
+#[test]
+fn test_enum_error() {
+    Python::with_gil(|py| {
         let dict = PyDict::new(py);
-        let f = Foo::extract(dict.as_ref()).expect("Failed to extract Foo from dict");
+        let err = Foo::extract(dict.as_ref()).unwrap_err();
+        assert_eq!(
+            err.to_string(),
+            "\
+TypeError: failed to extract enum Foo ('TupleVar | StructVar | TransparentTuple | TransparentStructVar | StructVarGetAttrArg | StructWithGetItem | StructWithGetItemArg')
+- variant TupleVar (TupleVar): 'dict' object cannot be converted to 'PyTuple'
+- variant StructVar (StructVar): 'dict' object has no attribute 'test'
+- variant TransparentTuple (TransparentTuple): 'dict' object cannot be interpreted as an integer
+- variant TransparentStructVar (TransparentStructVar): failed to extract field Foo :: TransparentStructVar.a
+- variant StructVarGetAttrArg (StructVarGetAttrArg): 'dict' object has no attribute 'bla'
+- variant StructWithGetItem (StructWithGetItem): 'a'
+- variant StructWithGetItemArg (StructWithGetItemArg): 'foo'"
+        );
+    });
+}
+
+#[derive(Debug, FromPyObject)]
+enum EnumWithCatchAll<'a> {
+    #[pyo3(transparent)]
+    Foo(Foo<'a>),
+    #[pyo3(transparent)]
+    CatchAll(&'a PyAny),
+}
+
+#[test]
+fn test_enum_catch_all() {
+    Python::with_gil(|py| {
+        let dict = PyDict::new(py);
+        let f = EnumWithCatchAll::extract(dict.as_ref())
+            .expect("Failed to extract EnumWithCatchAll from dict");
         match f {
-            Foo::CatchAll(any) => {
+            EnumWithCatchAll::CatchAll(any) => {
                 let d = <&PyDict>::extract(any).expect("Expected pydict");
                 assert!(d.is_empty());
             }
-            _ => panic!("Expected extracting Foo::CatchAll, got {:?}", f),
+            _ => panic!(
+                "Expected extracting EnumWithCatchAll::CatchAll, got {:?}",
+                f
+            ),
         }
     });
 }
@@ -409,10 +446,11 @@ fn test_err_rename() {
         assert!(f.is_err());
         assert_eq!(
             f.unwrap_err().to_string(),
-            "TypeError: failed to extract enum Bar (\'str | uint | int\')\n- variant A (str): \
-         \'dict\' object cannot be converted to \'PyString\'\n- variant B (uint): \'dict\' object \
-         cannot be interpreted as an integer\n- variant C (int): \'dict\' object cannot be \
-         interpreted as an integer\n"
+            "\
+TypeError: failed to extract enum Bar (\'str | uint | int\')
+- variant A (str): \'dict\' object cannot be converted to \'PyString\'
+- variant B (uint): \'dict\' object cannot be interpreted as an integer
+- variant C (int): \'dict\' object cannot be interpreted as an integer"
         );
     });
 }
@@ -443,3 +481,40 @@ fn test_from_py_with() {
         assert_eq!(zap.some_object_length, 3usize);
     });
 }
+
+#[derive(Debug, FromPyObject)]
+pub struct ZapTuple(String, #[pyo3(from_py_with = "PyAny::len")] usize);
+
+#[test]
+fn test_from_py_with_tuple_struct() {
+    Python::with_gil(|py| {
+        let py_zap = py
+            .eval(r#"("whatever", [1, 2, 3])"#, None, None)
+            .expect("failed to create tuple");
+
+        let zap = ZapTuple::extract(py_zap).unwrap();
+
+        assert_eq!(zap.0, "whatever");
+        assert_eq!(zap.1, 3usize);
+    });
+}
+
+#[derive(Debug, FromPyObject, PartialEq)]
+pub enum ZapEnum {
+    Zip(#[pyo3(from_py_with = "PyAny::len")] usize),
+    Zap(String, #[pyo3(from_py_with = "PyAny::len")] usize),
+}
+
+#[test]
+fn test_from_py_with_enum() {
+    Python::with_gil(|py| {
+        let py_zap = py
+            .eval(r#"("whatever", [1, 2, 3])"#, None, None)
+            .expect("failed to create tuple");
+
+        let zap = ZapEnum::extract(py_zap).unwrap();
+        let expected_zap = ZapEnum::Zap(String::from("whatever"), 3usize);
+
+        assert_eq!(zap, expected_zap);
+    });
+}
