@@ -1,3 +1,5 @@
+#![cfg(feature = "macros")]
+
 use pyo3::prelude::*;
 use pyo3::py_run;
 use pyo3::types::{IntoPyDict, PyDict, PyList, PySet, PyString, PyTuple, PyType};
@@ -840,12 +842,12 @@ impl r#RawIdents {
         r#type: PyObject,
         r#subtype: PyObject,
         r#subsubtype: PyObject,
-    ) -> PyResult<Self> {
-        Ok(Self {
+    ) -> Self {
+        Self {
             r#type,
             r#subtype,
             r#subsubtype,
-        })
+        }
     }
 
     #[getter(r#subtype)]
